# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/data.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2
from cogment.api import environment_pb2 as cogment_dot_api_dot_environment__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/data.proto',
  package='cogment',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x16\x63ogment/api/data.proto\x12\x07\x63ogment\x1a\x18\x63ogment/api/common.proto\x1a\x1d\x63ogment/api/environment.proto\"k\n\nDatalogMsg\x12,\n\x0ctrial_params\x18\x01 \x01(\x0b\x32\x14.cogment.TrialParamsH\x00\x12(\n\x06sample\x18\x02 \x01(\x0b\x32\x16.cogment.DatalogSampleH\x00\x42\x05\n\x03msg\"\xe6\x01\n\rDatalogSample\x12\x11\n\ttimestamp\x18\x07 \x01(\x06\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x05 \x01(\t\x12-\n\x0cobservations\x18\x02 \x01(\x0b\x32\x17.cogment.ObservationSet\x12 \n\x07\x61\x63tions\x18\x03 \x03(\x0b\x32\x0f.cogment.Action\x12 \n\x07rewards\x18\x04 \x03(\x0b\x32\x0f.cogment.Reward\x12,\n\x08messages\x18\x06 \x03(\x0b\x32\x1a.cogment.MessageCollection\"\n\n\x08LogReply2@\n\x0bLogExporter\x12\x31\n\x03Log\x12\x13.cogment.DatalogMsg\x1a\x11.cogment.LogReply\"\x00(\x01\x62\x06proto3')
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,cogment_dot_api_dot_environment__pb2.DESCRIPTOR,])




_DATALOGMSG = _descriptor.Descriptor(
  name='DatalogMsg',
  full_name='cogment.DatalogMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_params', full_name='cogment.DatalogMsg.trial_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sample', full_name='cogment.DatalogMsg.sample', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='msg', full_name='cogment.DatalogMsg.msg',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=92,
  serialized_end=199,
)


_DATALOGSAMPLE = _descriptor.Descriptor(
  name='DatalogSample',
  full_name='cogment.DatalogSample',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogment.DatalogSample.timestamp', index=0,
      number=7, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.DatalogSample.trial_id', index=1,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='cogment.DatalogSample.user_id', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='observations', full_name='cogment.DatalogSample.observations', index=3,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actions', full_name='cogment.DatalogSample.actions', index=4,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rewards', full_name='cogment.DatalogSample.rewards', index=5,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.DatalogSample.messages', index=6,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=202,
  serialized_end=432,
)


_LOGREPLY = _descriptor.Descriptor(
  name='LogReply',
  full_name='cogment.LogReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=434,
  serialized_end=444,
)

_DATALOGMSG.fields_by_name['trial_params'].message_type = cogment_dot_api_dot_common__pb2._TRIALPARAMS
_DATALOGMSG.fields_by_name['sample'].message_type = _DATALOGSAMPLE
_DATALOGMSG.oneofs_by_name['msg'].fields.append(
  _DATALOGMSG.fields_by_name['trial_params'])
_DATALOGMSG.fields_by_name['trial_params'].containing_oneof = _DATALOGMSG.oneofs_by_name['msg']
_DATALOGMSG.oneofs_by_name['msg'].fields.append(
  _DATALOGMSG.fields_by_name['sample'])
_DATALOGMSG.fields_by_name['sample'].containing_oneof = _DATALOGMSG.oneofs_by_name['msg']
_DATALOGSAMPLE.fields_by_name['observations'].message_type = cogment_dot_api_dot_environment__pb2._OBSERVATIONSET
_DATALOGSAMPLE.fields_by_name['actions'].message_type = cogment_dot_api_dot_common__pb2._ACTION
_DATALOGSAMPLE.fields_by_name['rewards'].message_type = cogment_dot_api_dot_common__pb2._REWARD
_DATALOGSAMPLE.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGECOLLECTION
DESCRIPTOR.message_types_by_name['DatalogMsg'] = _DATALOGMSG
DESCRIPTOR.message_types_by_name['DatalogSample'] = _DATALOGSAMPLE
DESCRIPTOR.message_types_by_name['LogReply'] = _LOGREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatalogMsg = _reflection.GeneratedProtocolMessageType('DatalogMsg', (_message.Message,), {
  'DESCRIPTOR' : _DATALOGMSG,
  '__module__' : 'cogment.api.data_pb2'
  # @@protoc_insertion_point(class_scope:cogment.DatalogMsg)
  })
_sym_db.RegisterMessage(DatalogMsg)

DatalogSample = _reflection.GeneratedProtocolMessageType('DatalogSample', (_message.Message,), {
  'DESCRIPTOR' : _DATALOGSAMPLE,
  '__module__' : 'cogment.api.data_pb2'
  # @@protoc_insertion_point(class_scope:cogment.DatalogSample)
  })
_sym_db.RegisterMessage(DatalogSample)

LogReply = _reflection.GeneratedProtocolMessageType('LogReply', (_message.Message,), {
  'DESCRIPTOR' : _LOGREPLY,
  '__module__' : 'cogment.api.data_pb2'
  # @@protoc_insertion_point(class_scope:cogment.LogReply)
  })
_sym_db.RegisterMessage(LogReply)



_LOGEXPORTER = _descriptor.ServiceDescriptor(
  name='LogExporter',
  full_name='cogment.LogExporter',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=446,
  serialized_end=510,
  methods=[
  _descriptor.MethodDescriptor(
    name='Log',
    full_name='cogment.LogExporter.Log',
    index=0,
    containing_service=None,
    input_type=_DATALOGMSG,
    output_type=_LOGREPLY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_LOGEXPORTER)

DESCRIPTOR.services_by_name['LogExporter'] = _LOGEXPORTER

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/agent.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/agent.proto',
  package='cogment',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x17\x63ogment/api/agent.proto\x12\x07\x63ogment\x1a\x18\x63ogment/api/common.proto\"\x9f\x01\n\x11\x41gentStartRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12$\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x14.cogment.ActorConfig\x12*\n\x0ctrial_config\x18\x05 \x01(\x0b\x32\x14.cogment.TrialConfig\x12\x14\n\x0c\x61\x63tor_counts\x18\x04 \x03(\x05\"\x11\n\x0f\x41gentStartReply\"]\n\x0f\x41gentEndRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12&\n\nfinal_data\x18\x03 \x01(\x0b\x32\x12.cogment.FinalData\"\x0f\n\rAgentEndReply\"c\n\x12\x41gentDecideRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12)\n\x0bobservation\x18\x03 \x01(\x0b\x32\x14.cogment.Observation\"}\n\x10\x41gentDecideReply\x12\x1f\n\x06\x61\x63tion\x18\x01 \x01(\x0b\x32\x0f.cogment.Action\x12$\n\tfeedbacks\x18\x02 \x03(\x0b\x32\x11.cogment.Feedback\x12\"\n\x08messages\x18\x03 \x03(\x0b\x32\x10.cogment.Message\"j\n\x12\x41gentRewardRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12\x0f\n\x07tick_id\x18\x04 \x01(\x05\x12\x1f\n\x06reward\x18\x03 \x01(\x0b\x32\x0f.cogment.Reward\"\x12\n\x10\x41gentRewardReply\"_\n\x15\x41gentOnMessageRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12\"\n\x08messages\x18\x03 \x03(\x0b\x32\x10.cogment.Message\"\x15\n\x13\x41gentOnMessageReply2\x94\x03\n\x05\x41gent\x12?\n\x05Start\x12\x1a.cogment.AgentStartRequest\x1a\x18.cogment.AgentStartReply\"\x00\x12\x39\n\x03\x45nd\x12\x18.cogment.AgentEndRequest\x1a\x16.cogment.AgentEndReply\"\x00\x12\x42\n\x06\x44\x65\x63ide\x12\x1b.cogment.AgentDecideRequest\x1a\x19.cogment.AgentDecideReply\"\x00\x12\x42\n\x06Reward\x12\x1b.cogment.AgentRewardRequest\x1a\x19.cogment.AgentRewardReply\"\x00\x12:\n\x07Version\x12\x17.cogment.VersionRequest\x1a\x14.cogment.VersionInfo\"\x00\x12K\n\tOnMessage\x12\x1e.cogment.AgentOnMessageRequest\x1a\x1c.cogment.AgentOnMessageReply\"\x00\x62\x06proto3')
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,])




_AGENTSTARTREQUEST = _descriptor.Descriptor(
  name='AgentStartRequest',
  full_name='cogment.AgentStartRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.AgentStartRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.AgentStartRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogment.AgentStartRequest.config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trial_config', full_name='cogment.AgentStartRequest.trial_config', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_counts', full_name='cogment.AgentStartRequest.actor_counts', index=4,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=63,
  serialized_end=222,
)


_AGENTSTARTREPLY = _descriptor.Descriptor(
  name='AgentStartReply',
  full_name='cogment.AgentStartReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=224,
  serialized_end=241,
)


_AGENTENDREQUEST = _descriptor.Descriptor(
  name='AgentEndRequest',
  full_name='cogment.AgentEndRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.AgentEndRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.AgentEndRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='final_data', full_name='cogment.AgentEndRequest.final_data', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=243,
  serialized_end=336,
)


_AGENTENDREPLY = _descriptor.Descriptor(
  name='AgentEndReply',
  full_name='cogment.AgentEndReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=338,
  serialized_end=353,
)


_AGENTDECIDEREQUEST = _descriptor.Descriptor(
  name='AgentDecideRequest',
  full_name='cogment.AgentDecideRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.AgentDecideRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.AgentDecideRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='observation', full_name='cogment.AgentDecideRequest.observation', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=355,
  serialized_end=454,
)


_AGENTDECIDEREPLY = _descriptor.Descriptor(
  name='AgentDecideReply',
  full_name='cogment.AgentDecideReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='cogment.AgentDecideReply.action', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feedbacks', full_name='cogment.AgentDecideReply.feedbacks', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.AgentDecideReply.messages', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=456,
  serialized_end=581,
)


_AGENTREWARDREQUEST = _descriptor.Descriptor(
  name='AgentRewardRequest',
  full_name='cogment.AgentRewardRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.AgentRewardRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.AgentRewardRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogment.AgentRewardRequest.tick_id', index=2,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reward', full_name='cogment.AgentRewardRequest.reward', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=583,
  serialized_end=689,
)


_AGENTREWARDREPLY = _descriptor.Descriptor(
  name='AgentRewardReply',
  full_name='cogment.AgentRewardReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=691,
  serialized_end=709,
)


_AGENTONMESSAGEREQUEST = _descriptor.Descriptor(
  name='AgentOnMessageRequest',
  full_name='cogment.AgentOnMessageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.AgentOnMessageRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.AgentOnMessageRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.AgentOnMessageRequest.messages', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=711,
  serialized_end=806,
)


_AGENTONMESSAGEREPLY = _descriptor.Descriptor(
  name='AgentOnMessageReply',
  full_name='cogment.AgentOnMessageReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=808,
  serialized_end=829,
)

_AGENTSTARTREQUEST.fields_by_name['config'].message_type = cogment_dot_api_dot_common__pb2._ACTORCONFIG
_AGENTSTARTREQUEST.fields_by_name['trial_config'].message_type = cogment_dot_api_dot_common__pb2._TRIALCONFIG
_AGENTENDREQUEST.fields_by_name['final_data'].message_type = cogment_dot_api_dot_common__pb2._FINALDATA
_AGENTDECIDEREQUEST.fields_by_name['observation'].message_type = cogment_dot_api_dot_common__pb2._OBSERVATION
_AGENTDECIDEREPLY.fields_by_name['action'].message_type = cogment_dot_api_dot_common__pb2._ACTION
_AGENTDECIDEREPLY.fields_by_name['feedbacks'].message_type = cogment_dot_api_dot_common__pb2._FEEDBACK
_AGENTDECIDEREPLY.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
_AGENTREWARDREQUEST.fields_by_name['reward'].message_type = cogment_dot_api_dot_common__pb2._REWARD
_AGENTONMESSAGEREQUEST.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
DESCRIPTOR.message_types_by_name['AgentStartRequest'] = _AGENTSTARTREQUEST
DESCRIPTOR.message_types_by_name['AgentStartReply'] = _AGENTSTARTREPLY
DESCRIPTOR.message_types_by_name['AgentEndRequest'] = _AGENTENDREQUEST
DESCRIPTOR.message_types_by_name['AgentEndReply'] = _AGENTENDREPLY
DESCRIPTOR.message_types_by_name['AgentDecideRequest'] = _AGENTDECIDEREQUEST
DESCRIPTOR.message_types_by_name['AgentDecideReply'] = _AGENTDECIDEREPLY
DESCRIPTOR.message_types_by_name['AgentRewardRequest'] = _AGENTREWARDREQUEST
DESCRIPTOR.message_types_by_name['AgentRewardReply'] = _AGENTREWARDREPLY
DESCRIPTOR.message_types_by_name['AgentOnMessageRequest'] = _AGENTONMESSAGEREQUEST
DESCRIPTOR.message_types_by_name['AgentOnMessageReply'] = _AGENTONMESSAGEREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AgentStartRequest = _reflection.GeneratedProtocolMessageType('AgentStartRequest', (_message.Message,), {
  'DESCRIPTOR' : _AGENTSTARTREQUEST,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentStartRequest)
  })
_sym_db.RegisterMessage(AgentStartRequest)

AgentStartReply = _reflection.GeneratedProtocolMessageType('AgentStartReply', (_message.Message,), {
  'DESCRIPTOR' : _AGENTSTARTREPLY,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentStartReply)
  })
_sym_db.RegisterMessage(AgentStartReply)

AgentEndRequest = _reflection.GeneratedProtocolMessageType('AgentEndRequest', (_message.Message,), {
  'DESCRIPTOR' : _AGENTENDREQUEST,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentEndRequest)
  })
_sym_db.RegisterMessage(AgentEndRequest)

AgentEndReply = _reflection.GeneratedProtocolMessageType('AgentEndReply', (_message.Message,), {
  'DESCRIPTOR' : _AGENTENDREPLY,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentEndReply)
  })
_sym_db.RegisterMessage(AgentEndReply)

AgentDecideRequest = _reflection.GeneratedProtocolMessageType('AgentDecideRequest', (_message.Message,), {
  'DESCRIPTOR' : _AGENTDECIDEREQUEST,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentDecideRequest)
  })
_sym_db.RegisterMessage(AgentDecideRequest)

AgentDecideReply = _reflection.GeneratedProtocolMessageType('AgentDecideReply', (_message.Message,), {
  'DESCRIPTOR' : _AGENTDECIDEREPLY,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentDecideReply)
  })
_sym_db.RegisterMessage(AgentDecideReply)

AgentRewardRequest = _reflection.GeneratedProtocolMessageType('AgentRewardRequest', (_message.Message,), {
  'DESCRIPTOR' : _AGENTREWARDREQUEST,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentRewardRequest)
  })
_sym_db.RegisterMessage(AgentRewardRequest)

AgentRewardReply = _reflection.GeneratedProtocolMessageType('AgentRewardReply', (_message.Message,), {
  'DESCRIPTOR' : _AGENTREWARDREPLY,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentRewardReply)
  })
_sym_db.RegisterMessage(AgentRewardReply)

AgentOnMessageRequest = _reflection.GeneratedProtocolMessageType('AgentOnMessageRequest', (_message.Message,), {
  'DESCRIPTOR' : _AGENTONMESSAGEREQUEST,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentOnMessageRequest)
  })
_sym_db.RegisterMessage(AgentOnMessageRequest)

AgentOnMessageReply = _reflection.GeneratedProtocolMessageType('AgentOnMessageReply', (_message.Message,), {
  'DESCRIPTOR' : _AGENTONMESSAGEREPLY,
  '__module__' : 'cogment.api.agent_pb2'
  # @@protoc_insertion_point(class_scope:cogment.AgentOnMessageReply)
  })
_sym_db.RegisterMessage(AgentOnMessageReply)



_AGENT = _descriptor.ServiceDescriptor(
  name='Agent',
  full_name='cogment.Agent',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=832,
  serialized_end=1236,
  methods=[
  _descriptor.MethodDescriptor(
    name='Start',
    full_name='cogment.Agent.Start',
    index=0,
    containing_service=None,
    input_type=_AGENTSTARTREQUEST,
    output_type=_AGENTSTARTREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='End',
    full_name='cogment.Agent.End',
    index=1,
    containing_service=None,
    input_type=_AGENTENDREQUEST,
    output_type=_AGENTENDREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Decide',
    full_name='cogment.Agent.Decide',
    index=2,
    containing_service=None,
    input_type=_AGENTDECIDEREQUEST,
    output_type=_AGENTDECIDEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Reward',
    full_name='cogment.Agent.Reward',
    index=3,
    containing_service=None,
    input_type=_AGENTREWARDREQUEST,
    output_type=_AGENTREWARDREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='cogment.Agent.Version',
    index=4,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._VERSIONREQUEST,
    output_type=cogment_dot_api_dot_common__pb2._VERSIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OnMessage',
    full_name='cogment.Agent.OnMessage',
    index=5,
    containing_service=None,
    input_type=_AGENTONMESSAGEREQUEST,
    output_type=_AGENTONMESSAGEREPLY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_AGENT)

DESCRIPTOR.services_by_name['Agent'] = _AGENT

# @@protoc_insertion_point(module_scope)



from types import SimpleNamespace
from cogment.api.common_pb2 import Feedback, Message


class Actor:

    def __init__(self, actor_class, actor_id, id_in_class, trial):
        self.actor_class = actor_class
        self.actor_id = actor_id
        self.id_in_class = id_in_class

        self._feedback = []
        self._message = []
        self.trial = trial

    def add_feedback(self, value=None, confidence=None, tick_id=None, user_data=None):
        if tick_id is None:
            tick_id = -1

        self._feedback.append((tick_id, value, confidence, user_data))

    def send_message(self, user_data=None):
        self._message.append(user_data)


class Env:

    def __init__(self, actor_id, trial):
        self.actor_id = actor_id

        self._message = []
        self.trial = trial

    def send_message(self, user_data):
        self._message.append((user_data))


class Trial:

    def __init__(self, id, settings, actor_counts, trial_config):
        self.id = id
        self.actor_counts = actor_counts
        self.actors = SimpleNamespace(all=[])
        self.settings = settings
        self.tick_id = 0
        self.trial_config = trial_config

        self.env = SimpleNamespace(all=[])

        actor_id = 0
        for class_index, actor_class in enumerate(self.settings.actor_classes):
            actor_list = []
            id_in_class = 0
            for _ in range(actor_counts[class_index]):
                actor = Actor(actor_class, actor_id, id_in_class, self)
                actor_list.append(actor)
                self.actors.all.append(actor)
                actor_id += 1
                id_in_class += 1

            setattr(self.actors, actor_class.id, actor_list)

        env_list = []
        env = Env(-1, self)
        env_list.append(env)
        self.env.all.append(env)
        env_class = self.settings.env_class

        setattr(self.env, env_class.id, env_list)

    def new_observation_table(self):
        return self.settings.ObservationsTable(self.actor_counts)

    def _get_all_feedback(self):
        for actor in self.actors.all:
            a_fb = actor._feedback
            actor._feedback = []

            for fb in a_fb:
                re = Feedback(
                    actor_id=actor.actor_id,
                    tick_id=fb[0],
                    value=fb[1],
                    confidence=fb[2],
                    agent_id=actor.id_in_class
                )
                if fb[3] is not None:
                    re.content = fb[3].SerializeToString()

                yield re

    def _get_all_messages(self, source_id):

        for actor in self.actors.all:
            a_msg = actor._message
            actor._message = []

            for msg in a_msg:
                re = Message(
                    sender_id=source_id,
                    receiver_id=actor.actor_id
                )
                if msg is not None:
                    re.payload.Pack(msg)
                yield re

        for env in self.env.all:
            e_msg = env._message
            env._message = []

            for msg in e_msg:
                re = Message(
                    sender_id=source_id,
                    receiver_id=-1
                )

                if msg is not None:
                    re.payload.Pack(msg)
                yield re

    def multi_cast(self, user_data=None, send_list=None):

        if send_list is not None:
            possible_targets = [i for i in range(sum(self.actor_counts))]
            possible_targets.append(-1)

            for target in send_list:
                if target in possible_targets:
                    if target == -1:
                        self.env.env[0].send_message(user_data=user_data)
                    elif target == 0:
                        self.actors.human[0].send_message(user_data=user_data)
                    else:
                        self.actors.agent[
                            target - 1].send_message(user_data=user_data)
                else:
                    raise Exception(target, "is not a valid agent!!")



# Copyright 2019 Age of Minds inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cogment.env_service import Environment
from cogment.agent_service import Agent
from cogment.hooks_service import TrialHooks
from cogment.grpc_server import GrpcServer
from cogment.actor_class import ActorClass
from cogment.trial import Trial
from cogment.trial import Actor
from cogment.env_class import EnvClass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/common.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/common.proto',
  package='cogment',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x18\x63ogment/api/common.proto\x12\x07\x63ogment\x1a\x19google/protobuf/any.proto\"\x10\n\x0eVersionRequest\"g\n\x0bVersionInfo\x12.\n\x08versions\x18\x01 \x03(\x0b\x32\x1c.cogment.VersionInfo.Version\x1a(\n\x07Version\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"\x1e\n\x0bTrialConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"\x1e\n\x0b\x41\x63torConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"$\n\x11\x45nvironmentConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"Q\n\x11\x45nvironmentParams\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12*\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x1a.cogment.EnvironmentConfig\"Z\n\x0b\x41\x63torParams\x12\x13\n\x0b\x61\x63tor_class\x18\x01 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x02 \x01(\t\x12$\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x14.cogment.ActorConfig\"\xbb\x01\n\x0bTrialParams\x12*\n\x0ctrial_config\x18\x01 \x01(\x0b\x32\x14.cogment.TrialConfig\x12/\n\x0b\x65nvironment\x18\x02 \x01(\x0b\x32\x1a.cogment.EnvironmentParams\x12$\n\x06\x61\x63tors\x18\x03 \x03(\x0b\x32\x14.cogment.ActorParams\x12\x11\n\tmax_steps\x18\x04 \x01(\r\x12\x16\n\x0emax_inactivity\x18\x05 \x01(\r\"4\n\x0fObservationData\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\x12\x10\n\x08snapshot\x18\x02 \x01(\x08\"Y\n\x0bObservation\x12\x0f\n\x07tick_id\x18\x01 \x01(\x04\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12&\n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32\x18.cogment.ObservationData\"\x19\n\x06\x41\x63tion\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"s\n\x08\x46\x65\x65\x64\x62\x61\x63k\x12\x10\n\x08\x61\x63tor_id\x18\x01 \x01(\r\x12\x0f\n\x07tick_id\x18\x02 \x01(\x05\x12\r\n\x05value\x18\x03 \x01(\x02\x12\x12\n\nconfidence\x18\x04 \x01(\x02\x12\x0f\n\x07\x63ontent\x18\x05 \x01(\x0c\x12\x10\n\x08\x61gent_id\x18\x07 \x01(\r\"Q\n\x06Reward\x12\r\n\x05value\x18\x01 \x01(\x02\x12\x12\n\nconfidence\x18\x02 \x01(\x02\x12$\n\tfeedbacks\x18\x03 \x03(\x0b\x32\x11.cogment.Feedback\"X\n\x07Message\x12\x11\n\tsender_id\x18\x01 \x01(\x05\x12\x13\n\x0breceiver_id\x18\x02 \x01(\x05\x12%\n\x07payload\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\"7\n\x11MessageCollection\x12\"\n\x08messages\x18\x01 \x03(\x0b\x32\x10.cogment.Message\"}\n\tFinalData\x12*\n\x0cobservations\x18\x01 \x03(\x0b\x32\x14.cogment.Observation\x12 \n\x07rewards\x18\x02 \x03(\x0b\x32\x0f.cogment.Reward\x12\"\n\x08messages\x18\x03 \x03(\x0b\x32\x10.cogment.Messageb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_VERSIONREQUEST = _descriptor.Descriptor(
  name='VersionRequest',
  full_name='cogment.VersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=80,
)


_VERSIONINFO_VERSION = _descriptor.Descriptor(
  name='Version',
  full_name='cogment.VersionInfo.Version',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='cogment.VersionInfo.Version.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='cogment.VersionInfo.Version.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=145,
  serialized_end=185,
)

_VERSIONINFO = _descriptor.Descriptor(
  name='VersionInfo',
  full_name='cogment.VersionInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='versions', full_name='cogment.VersionInfo.versions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_VERSIONINFO_VERSION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=82,
  serialized_end=185,
)


_TRIALCONFIG = _descriptor.Descriptor(
  name='TrialConfig',
  full_name='cogment.TrialConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogment.TrialConfig.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=187,
  serialized_end=217,
)


_ACTORCONFIG = _descriptor.Descriptor(
  name='ActorConfig',
  full_name='cogment.ActorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogment.ActorConfig.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=219,
  serialized_end=249,
)


_ENVIRONMENTCONFIG = _descriptor.Descriptor(
  name='EnvironmentConfig',
  full_name='cogment.EnvironmentConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogment.EnvironmentConfig.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=251,
  serialized_end=287,
)


_ENVIRONMENTPARAMS = _descriptor.Descriptor(
  name='EnvironmentParams',
  full_name='cogment.EnvironmentParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='cogment.EnvironmentParams.endpoint', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogment.EnvironmentParams.config', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=289,
  serialized_end=370,
)


_ACTORPARAMS = _descriptor.Descriptor(
  name='ActorParams',
  full_name='cogment.ActorParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='actor_class', full_name='cogment.ActorParams.actor_class', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='cogment.ActorParams.endpoint', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogment.ActorParams.config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=372,
  serialized_end=462,
)


_TRIALPARAMS = _descriptor.Descriptor(
  name='TrialParams',
  full_name='cogment.TrialParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_config', full_name='cogment.TrialParams.trial_config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='environment', full_name='cogment.TrialParams.environment', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actors', full_name='cogment.TrialParams.actors', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_steps', full_name='cogment.TrialParams.max_steps', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_inactivity', full_name='cogment.TrialParams.max_inactivity', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=465,
  serialized_end=652,
)


_OBSERVATIONDATA = _descriptor.Descriptor(
  name='ObservationData',
  full_name='cogment.ObservationData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogment.ObservationData.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='snapshot', full_name='cogment.ObservationData.snapshot', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=654,
  serialized_end=706,
)


_OBSERVATION = _descriptor.Descriptor(
  name='Observation',
  full_name='cogment.Observation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogment.Observation.tick_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogment.Observation.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='cogment.Observation.data', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=708,
  serialized_end=797,
)


_ACTION = _descriptor.Descriptor(
  name='Action',
  full_name='cogment.Action',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogment.Action.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=799,
  serialized_end=824,
)


_FEEDBACK = _descriptor.Descriptor(
  name='Feedback',
  full_name='cogment.Feedback',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.Feedback.actor_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogment.Feedback.tick_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='cogment.Feedback.value', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='cogment.Feedback.confidence', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='content', full_name='cogment.Feedback.content', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='agent_id', full_name='cogment.Feedback.agent_id', index=5,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=826,
  serialized_end=941,
)


_REWARD = _descriptor.Descriptor(
  name='Reward',
  full_name='cogment.Reward',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='cogment.Reward.value', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='cogment.Reward.confidence', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feedbacks', full_name='cogment.Reward.feedbacks', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=943,
  serialized_end=1024,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='cogment.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender_id', full_name='cogment.Message.sender_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver_id', full_name='cogment.Message.receiver_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='cogment.Message.payload', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1026,
  serialized_end=1114,
)


_MESSAGECOLLECTION = _descriptor.Descriptor(
  name='MessageCollection',
  full_name='cogment.MessageCollection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.MessageCollection.messages', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1116,
  serialized_end=1171,
)


_FINALDATA = _descriptor.Descriptor(
  name='FinalData',
  full_name='cogment.FinalData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='observations', full_name='cogment.FinalData.observations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rewards', full_name='cogment.FinalData.rewards', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.FinalData.messages', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1173,
  serialized_end=1298,
)

_VERSIONINFO_VERSION.containing_type = _VERSIONINFO
_VERSIONINFO.fields_by_name['versions'].message_type = _VERSIONINFO_VERSION
_ENVIRONMENTPARAMS.fields_by_name['config'].message_type = _ENVIRONMENTCONFIG
_ACTORPARAMS.fields_by_name['config'].message_type = _ACTORCONFIG
_TRIALPARAMS.fields_by_name['trial_config'].message_type = _TRIALCONFIG
_TRIALPARAMS.fields_by_name['environment'].message_type = _ENVIRONMENTPARAMS
_TRIALPARAMS.fields_by_name['actors'].message_type = _ACTORPARAMS
_OBSERVATION.fields_by_name['data'].message_type = _OBSERVATIONDATA
_REWARD.fields_by_name['feedbacks'].message_type = _FEEDBACK
_MESSAGE.fields_by_name['payload'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_MESSAGECOLLECTION.fields_by_name['messages'].message_type = _MESSAGE
_FINALDATA.fields_by_name['observations'].message_type = _OBSERVATION
_FINALDATA.fields_by_name['rewards'].message_type = _REWARD
_FINALDATA.fields_by_name['messages'].message_type = _MESSAGE
DESCRIPTOR.message_types_by_name['VersionRequest'] = _VERSIONREQUEST
DESCRIPTOR.message_types_by_name['VersionInfo'] = _VERSIONINFO
DESCRIPTOR.message_types_by_name['TrialConfig'] = _TRIALCONFIG
DESCRIPTOR.message_types_by_name['ActorConfig'] = _ACTORCONFIG
DESCRIPTOR.message_types_by_name['EnvironmentConfig'] = _ENVIRONMENTCONFIG
DESCRIPTOR.message_types_by_name['EnvironmentParams'] = _ENVIRONMENTPARAMS
DESCRIPTOR.message_types_by_name['ActorParams'] = _ACTORPARAMS
DESCRIPTOR.message_types_by_name['TrialParams'] = _TRIALPARAMS
DESCRIPTOR.message_types_by_name['ObservationData'] = _OBSERVATIONDATA
DESCRIPTOR.message_types_by_name['Observation'] = _OBSERVATION
DESCRIPTOR.message_types_by_name['Action'] = _ACTION
DESCRIPTOR.message_types_by_name['Feedback'] = _FEEDBACK
DESCRIPTOR.message_types_by_name['Reward'] = _REWARD
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.message_types_by_name['MessageCollection'] = _MESSAGECOLLECTION
DESCRIPTOR.message_types_by_name['FinalData'] = _FINALDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VersionRequest = _reflection.GeneratedProtocolMessageType('VersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERSIONREQUEST,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.VersionRequest)
  })
_sym_db.RegisterMessage(VersionRequest)

VersionInfo = _reflection.GeneratedProtocolMessageType('VersionInfo', (_message.Message,), {

  'Version' : _reflection.GeneratedProtocolMessageType('Version', (_message.Message,), {
    'DESCRIPTOR' : _VERSIONINFO_VERSION,
    '__module__' : 'cogment.api.common_pb2'
    # @@protoc_insertion_point(class_scope:cogment.VersionInfo.Version)
    })
  ,
  'DESCRIPTOR' : _VERSIONINFO,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.VersionInfo)
  })
_sym_db.RegisterMessage(VersionInfo)
_sym_db.RegisterMessage(VersionInfo.Version)

TrialConfig = _reflection.GeneratedProtocolMessageType('TrialConfig', (_message.Message,), {
  'DESCRIPTOR' : _TRIALCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialConfig)
  })
_sym_db.RegisterMessage(TrialConfig)

ActorConfig = _reflection.GeneratedProtocolMessageType('ActorConfig', (_message.Message,), {
  'DESCRIPTOR' : _ACTORCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.ActorConfig)
  })
_sym_db.RegisterMessage(ActorConfig)

EnvironmentConfig = _reflection.GeneratedProtocolMessageType('EnvironmentConfig', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvironmentConfig)
  })
_sym_db.RegisterMessage(EnvironmentConfig)

EnvironmentParams = _reflection.GeneratedProtocolMessageType('EnvironmentParams', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvironmentParams)
  })
_sym_db.RegisterMessage(EnvironmentParams)

ActorParams = _reflection.GeneratedProtocolMessageType('ActorParams', (_message.Message,), {
  'DESCRIPTOR' : _ACTORPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.ActorParams)
  })
_sym_db.RegisterMessage(ActorParams)

TrialParams = _reflection.GeneratedProtocolMessageType('TrialParams', (_message.Message,), {
  'DESCRIPTOR' : _TRIALPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.TrialParams)
  })
_sym_db.RegisterMessage(TrialParams)

ObservationData = _reflection.GeneratedProtocolMessageType('ObservationData', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATIONDATA,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.ObservationData)
  })
_sym_db.RegisterMessage(ObservationData)

Observation = _reflection.GeneratedProtocolMessageType('Observation', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.Observation)
  })
_sym_db.RegisterMessage(Observation)

Action = _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
  'DESCRIPTOR' : _ACTION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.Action)
  })
_sym_db.RegisterMessage(Action)

Feedback = _reflection.GeneratedProtocolMessageType('Feedback', (_message.Message,), {
  'DESCRIPTOR' : _FEEDBACK,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.Feedback)
  })
_sym_db.RegisterMessage(Feedback)

Reward = _reflection.GeneratedProtocolMessageType('Reward', (_message.Message,), {
  'DESCRIPTOR' : _REWARD,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.Reward)
  })
_sym_db.RegisterMessage(Reward)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.Message)
  })
_sym_db.RegisterMessage(Message)

MessageCollection = _reflection.GeneratedProtocolMessageType('MessageCollection', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGECOLLECTION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.MessageCollection)
  })
_sym_db.RegisterMessage(MessageCollection)

FinalData = _reflection.GeneratedProtocolMessageType('FinalData', (_message.Message,), {
  'DESCRIPTOR' : _FINALDATA,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogment.FinalData)
  })
_sym_db.RegisterMessage(FinalData)


# @@protoc_insertion_point(module_scope)



from cogment.actor_class import ActorClass


# This is the default delta application function that is used when the
# Observation space and Observation delta of an actor is class is of the same
# type
def _apply_delta_replace(observation, delta):
    assert type(observation) == type(delta)
    return delta


# Correctly updates or creates an observation based on the data contained in a
# cogment.api.common_pb2.ObservationData
def DecodeObservationData(actor_class: ActorClass, data, previous_observation=None):
    if not previous_observation:
        previous_observation = actor_class.observation_space()

    if data.snapshot:
        previous_observation.ParseFromString(data.content)
        return previous_observation
    else:
        delta = actor_class.observation_delta()
        delta.ParseFromString(data.content)
        apply_fn = actor_class.observation_delta_apply_fn
        return apply_fn(previous_observation, delta)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from cogment.api import agent_pb2 as cogment_dot_api_dot_agent__pb2
from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


class AgentStub(object):
  """This is the connection between a model application and the orchestrator.
  The service is hosted on the model. 
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Start = channel.unary_unary(
        '/cogment.Agent/Start',
        request_serializer=cogment_dot_api_dot_agent__pb2.AgentStartRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_agent__pb2.AgentStartReply.FromString,
        )
    self.End = channel.unary_unary(
        '/cogment.Agent/End',
        request_serializer=cogment_dot_api_dot_agent__pb2.AgentEndRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_agent__pb2.AgentEndReply.FromString,
        )
    self.Decide = channel.unary_unary(
        '/cogment.Agent/Decide',
        request_serializer=cogment_dot_api_dot_agent__pb2.AgentDecideRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_agent__pb2.AgentDecideReply.FromString,
        )
    self.Reward = channel.unary_unary(
        '/cogment.Agent/Reward',
        request_serializer=cogment_dot_api_dot_agent__pb2.AgentRewardRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_agent__pb2.AgentRewardReply.FromString,
        )
    self.Version = channel.unary_unary(
        '/cogment.Agent/Version',
        request_serializer=cogment_dot_api_dot_common__pb2.VersionRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_common__pb2.VersionInfo.FromString,
        )
    self.OnMessage = channel.unary_unary(
        '/cogment.Agent/OnMessage',
        request_serializer=cogment_dot_api_dot_agent__pb2.AgentOnMessageRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_agent__pb2.AgentOnMessageReply.FromString,
        )


class AgentServicer(object):
  """This is the connection between a model application and the orchestrator.
  The service is hosted on the model. 
  """

  def Start(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def End(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Decide(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Reward(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Version(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def OnMessage(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_AgentServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Start': grpc.unary_unary_rpc_method_handler(
          servicer.Start,
          request_deserializer=cogment_dot_api_dot_agent__pb2.AgentStartRequest.FromString,
          response_serializer=cogment_dot_api_dot_agent__pb2.AgentStartReply.SerializeToString,
      ),
      'End': grpc.unary_unary_rpc_method_handler(
          servicer.End,
          request_deserializer=cogment_dot_api_dot_agent__pb2.AgentEndRequest.FromString,
          response_serializer=cogment_dot_api_dot_agent__pb2.AgentEndReply.SerializeToString,
      ),
      'Decide': grpc.unary_unary_rpc_method_handler(
          servicer.Decide,
          request_deserializer=cogment_dot_api_dot_agent__pb2.AgentDecideRequest.FromString,
          response_serializer=cogment_dot_api_dot_agent__pb2.AgentDecideReply.SerializeToString,
      ),
      'Reward': grpc.unary_unary_rpc_method_handler(
          servicer.Reward,
          request_deserializer=cogment_dot_api_dot_agent__pb2.AgentRewardRequest.FromString,
          response_serializer=cogment_dot_api_dot_agent__pb2.AgentRewardReply.SerializeToString,
      ),
      'Version': grpc.unary_unary_rpc_method_handler(
          servicer.Version,
          request_deserializer=cogment_dot_api_dot_common__pb2.VersionRequest.FromString,
          response_serializer=cogment_dot_api_dot_common__pb2.VersionInfo.SerializeToString,
      ),
      'OnMessage': grpc.unary_unary_rpc_method_handler(
          servicer.OnMessage,
          request_deserializer=cogment_dot_api_dot_agent__pb2.AgentOnMessageRequest.FromString,
          response_serializer=cogment_dot_api_dot_agent__pb2.AgentOnMessageReply.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'cogment.Agent', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2
from cogment.api import environment_pb2 as cogment_dot_api_dot_environment__pb2


class EnvironmentStub(object):
  """This is the connection between the orchestrator and the Env backend.
  This service is hosted on the backend executable, and contacted by the 
  orchestrator.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Start = channel.unary_unary(
        '/cogment.Environment/Start',
        request_serializer=cogment_dot_api_dot_environment__pb2.EnvStartRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_environment__pb2.EnvStartReply.FromString,
        )
    self.End = channel.unary_unary(
        '/cogment.Environment/End',
        request_serializer=cogment_dot_api_dot_environment__pb2.EnvEndRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_environment__pb2.EnvEndReply.FromString,
        )
    self.Update = channel.unary_unary(
        '/cogment.Environment/Update',
        request_serializer=cogment_dot_api_dot_environment__pb2.EnvUpdateRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_environment__pb2.EnvUpdateReply.FromString,
        )
    self.Version = channel.unary_unary(
        '/cogment.Environment/Version',
        request_serializer=cogment_dot_api_dot_common__pb2.VersionRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_common__pb2.VersionInfo.FromString,
        )
    self.OnMessage = channel.unary_unary(
        '/cogment.Environment/OnMessage',
        request_serializer=cogment_dot_api_dot_environment__pb2.EnvOnMessageRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_environment__pb2.EnvOnMessageReply.FromString,
        )


class EnvironmentServicer(object):
  """This is the connection between the orchestrator and the Env backend.
  This service is hosted on the backend executable, and contacted by the 
  orchestrator.
  """

  def Start(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def End(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Update(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Version(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def OnMessage(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_EnvironmentServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Start': grpc.unary_unary_rpc_method_handler(
          servicer.Start,
          request_deserializer=cogment_dot_api_dot_environment__pb2.EnvStartRequest.FromString,
          response_serializer=cogment_dot_api_dot_environment__pb2.EnvStartReply.SerializeToString,
      ),
      'End': grpc.unary_unary_rpc_method_handler(
          servicer.End,
          request_deserializer=cogment_dot_api_dot_environment__pb2.EnvEndRequest.FromString,
          response_serializer=cogment_dot_api_dot_environment__pb2.EnvEndReply.SerializeToString,
      ),
      'Update': grpc.unary_unary_rpc_method_handler(
          servicer.Update,
          request_deserializer=cogment_dot_api_dot_environment__pb2.EnvUpdateRequest.FromString,
          response_serializer=cogment_dot_api_dot_environment__pb2.EnvUpdateReply.SerializeToString,
      ),
      'Version': grpc.unary_unary_rpc_method_handler(
          servicer.Version,
          request_deserializer=cogment_dot_api_dot_common__pb2.VersionRequest.FromString,
          response_serializer=cogment_dot_api_dot_common__pb2.VersionInfo.SerializeToString,
      ),
      'OnMessage': grpc.unary_unary_rpc_method_handler(
          servicer.OnMessage,
          request_deserializer=cogment_dot_api_dot_environment__pb2.EnvOnMessageRequest.FromString,
          response_serializer=cogment_dot_api_dot_environment__pb2.EnvOnMessageReply.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'cogment.Environment', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



from cogment.api.common_pb2 import VersionInfo
from cogment.version import __version__

import grpc


def list_versions(cls):
    reply = VersionInfo()
    reply.versions.add(name='cogment_sdk', version=__version__)
    reply.versions.add(name='grpc', version=grpc.__version__)

    try:
        for name, version in cls.VERSIONS.items():
            reply.versions.add(name=name, version=version)
    except AttributeError as error:
        pass

    return reply



from uuid import uuid4
from queue import Queue
from threading import Thread

from cogment.api.orchestrator_pb2 import (
    TrialJoinRequest, TrialStartRequest, TrialFeedbackRequest, TrialActionRequest, TrialEndRequest, TrialMessageRequest)

from cogment.api.orchestrator_pb2_grpc import TrialStub
from cogment.api.common_pb2 import Action

from cogment.delta_encoding import DecodeObservationData

from cogment.trial import Trial
import grpc

import importlib

SESSIONS_ID_HEADER_NAME = 'session_id'


class ClientTrial(Trial):

    def __init__(self, conn, trial_start_rep, settings, actor_class,
                 actor_counts, initial_observation, session_id, trial_config):
        super().__init__(trial_start_rep.trial_id, settings, actor_counts, trial_config)
        self.connection = conn
        self.observation = initial_observation
        self.actor_id = trial_start_rep.actor_id
        self.actor_class = actor_class
        self.latest_reward = None
        self.latest_messages = None
        self.session_id = session_id
        self._end_callback = None
        self._action_queue = None
        self._updates_queue = None
        self._client_thread = None
        self._client_thread = None

    def _get_actions(self):
        while True:
            act = self._action_queue.get()
            if act is not None:
                yield TrialActionRequest(
                    trial_id=self.id,
                    actor_id=self.actor_id,
                    action=Action(content=act.SerializeToString()))
            else:
                break

    def _client_streaming_thread(self):
        metadata = ((SESSIONS_ID_HEADER_NAME, self.session_id),)
        updates = self.connection.stub.ActionStream(
            self._get_actions(), metadata=metadata)
        for update in updates:
            self._updates_queue.put(update)

    def begin_actions_stream(self):
        """This switches the client to streaming mode, using a single connection
        to transfer all actions and observations"""
        self._action_queue = Queue()
        self._updates_queue = Queue()
        self._client_thread = Thread(target=self._client_streaming_thread)
        self._client_thread.start()

    def set_on_trial_end_event_listener(self, callback):
        self._end_callback = callback

    # Perform an action on the trial, and advance time
    def do_action(self, action, on_message=None):
        self.flush_feedback()
        self.flush_message()

        update = None
        if self._action_queue:
            self._action_queue.put(action)
            update = self._updates_queue.get()
        else:
            # Send the update to the orchestrator
            update = self.connection.stub.Action(TrialActionRequest(
                trial_id=self.id,
                actor_id=self.actor_id,
                action=Action(content=action.SerializeToString())),
                metadata=((SESSIONS_ID_HEADER_NAME, self.session_id),))

        self.observation = DecodeObservationData(
            self.actor_class,
            update.observation.data,
            self.observation)

        self.latest_reward = update.reward
        self.latest_messages = update.messages

        self.tick_id = update.observation.tick_id

        if update.trial_is_over:
            if self._end_callback is not None:
                self._end_callback()

        if on_message is not None:
            for m in self.latest_messages.messages:
                class_type = m.payload.type_url.split('.')
                user_data = getattr(importlib.import_module(
                    self.settings.protolib), class_type[-1])()
                m.payload.Unpack(user_data)
                on_message(m.sender_id, user_data)

        # Return the latest observation
        return self.observation, self.latest_reward, self.latest_messages

    # Kill the trial
    def end(self):
        if self._action_queue:
            self._action_queue.put(None)
            self._client_thread.join()
        if self.plugins:
            # Inform plugins that the trial has ended
            for plugin in self.plugins:
                plugin.trial_ended()

        self.flush_feedback()
        self.flush_message()
        self.connection.stub.End(TrialEndRequest(trial_id=self.id),
                                 metadata=((SESSIONS_ID_HEADER_NAME, self.session_id),))

    def flush_feedback(self):
        feedbacks = list(self._get_all_feedback())

        if feedbacks:
            req = TrialFeedbackRequest(trial_id=self.id)

            req.feedbacks.extend(feedbacks)
            self.connection.stub.GiveFeedback(req, metadata=(
                (SESSIONS_ID_HEADER_NAME, self.session_id),))

    def flush_message(self):

        messages = list(self._get_all_messages(self.actor_id))

        if messages:
            req = TrialMessageRequest(trial_id=self.id)

            req.messages.extend(messages)
            self.connection.stub.SendMessage(req, metadata=(
                (SESSIONS_ID_HEADER_NAME, self.session_id),))


class _Connection_impl:

    def __init__(self, stub, settings):
        if not settings:
            raise Exception("missing settings")

        if not stub:
            raise Exception("missing grpc connection stub")

        self.stub = stub
        self.settings = settings

        self.__session_id = str(uuid4())

    def start_trial(self, actor_class, trial_cfg=None, plugins=[]):
        req = TrialStartRequest()

        if trial_cfg:
            req.config.content = trial_cfg.SerializeToString()

        rep = self.stub.Start(req, metadata=(
            (SESSIONS_ID_HEADER_NAME, self.__session_id),))

        observation = DecodeObservationData(actor_class, rep.observation.data)

        new_trial = ClientTrial(
            self, rep, self.settings, actor_class, rep.actor_counts,
            observation, self.__session_id, trial_cfg)

        # Start the plugins
        new_trial.plugins = [plugin.start_trial(
            new_trial) for plugin in plugins]
        return new_trial

    def join_trial(self, actor_class, trial_id, actor_id):
        req = TrialJoinRequest()
        req.trial_id = trial_id
        req.actor_id = actor_id

        rep = self.stub.Join(req, metadata=(
            (SESSIONS_ID_HEADER_NAME, self.__session_id),))

        observation = DecodeObservationData(actor_class, rep.observation.data)

        new_trial = ClientTrial(self, rep, self.settings, actor_class,
                                rep.actor_counts, observation, self.__session_id)

        return new_trial


class Connection(_Connection_impl):

    def __init__(self, settings, endpoint):

        channel = grpc.insecure_channel(endpoint)
        stub = TrialStub(channel)
        super().__init__(stub, settings)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2
from cogment.api import orchestrator_pb2 as cogment_dot_api_dot_orchestrator__pb2


class TrialStub(object):
  """This is the connection between a client application and the orchestrator.
  The service is hosted on the orchestrator itself. 
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GetTrialParams = channel.unary_unary(
        '/cogment.Trial/GetTrialParams',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.GetTrialParamsRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.GetTrialParamsReply.FromString,
        )
    self.Start = channel.unary_unary(
        '/cogment.Trial/Start',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialStartRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialStartReply.FromString,
        )
    self.Join = channel.unary_unary(
        '/cogment.Trial/Join',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialJoinRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialStartReply.FromString,
        )
    self.End = channel.unary_unary(
        '/cogment.Trial/End',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialEndRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialEndReply.FromString,
        )
    self.Action = channel.unary_unary(
        '/cogment.Trial/Action',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionReply.FromString,
        )
    self.ActionStream = channel.stream_stream(
        '/cogment.Trial/ActionStream',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionReply.FromString,
        )
    self.Heartbeat = channel.unary_unary(
        '/cogment.Trial/Heartbeat',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialHeartbeatRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialHeartbeatReply.FromString,
        )
    self.GiveFeedback = channel.unary_unary(
        '/cogment.Trial/GiveFeedback',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialFeedbackRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialFeedbackReply.FromString,
        )
    self.Version = channel.unary_unary(
        '/cogment.Trial/Version',
        request_serializer=cogment_dot_api_dot_common__pb2.VersionRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_common__pb2.VersionInfo.FromString,
        )
    self.SendMessage = channel.unary_unary(
        '/cogment.Trial/SendMessage',
        request_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialMessageRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialMessageReply.FromString,
        )


class TrialServicer(object):
  """This is the connection between a client application and the orchestrator.
  The service is hosted on the orchestrator itself. 
  """

  def GetTrialParams(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Start(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Join(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def End(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Action(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ActionStream(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Heartbeat(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GiveFeedback(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Version(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SendMessage(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_TrialServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GetTrialParams': grpc.unary_unary_rpc_method_handler(
          servicer.GetTrialParams,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.GetTrialParamsRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.GetTrialParamsReply.SerializeToString,
      ),
      'Start': grpc.unary_unary_rpc_method_handler(
          servicer.Start,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialStartRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialStartReply.SerializeToString,
      ),
      'Join': grpc.unary_unary_rpc_method_handler(
          servicer.Join,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialJoinRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialStartReply.SerializeToString,
      ),
      'End': grpc.unary_unary_rpc_method_handler(
          servicer.End,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialEndRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialEndReply.SerializeToString,
      ),
      'Action': grpc.unary_unary_rpc_method_handler(
          servicer.Action,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionReply.SerializeToString,
      ),
      'ActionStream': grpc.stream_stream_rpc_method_handler(
          servicer.ActionStream,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialActionReply.SerializeToString,
      ),
      'Heartbeat': grpc.unary_unary_rpc_method_handler(
          servicer.Heartbeat,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialHeartbeatRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialHeartbeatReply.SerializeToString,
      ),
      'GiveFeedback': grpc.unary_unary_rpc_method_handler(
          servicer.GiveFeedback,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialFeedbackRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialFeedbackReply.SerializeToString,
      ),
      'Version': grpc.unary_unary_rpc_method_handler(
          servicer.Version,
          request_deserializer=cogment_dot_api_dot_common__pb2.VersionRequest.FromString,
          response_serializer=cogment_dot_api_dot_common__pb2.VersionInfo.SerializeToString,
      ),
      'SendMessage': grpc.unary_unary_rpc_method_handler(
          servicer.SendMessage,
          request_deserializer=cogment_dot_api_dot_orchestrator__pb2.TrialMessageRequest.FromString,
          response_serializer=cogment_dot_api_dot_orchestrator__pb2.TrialMessageReply.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'cogment.Trial', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2
from cogment.api import hooks_pb2 as cogment_dot_api_dot_hooks__pb2


class TrialHooksStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.PreTrial = channel.unary_unary(
        '/cogment.TrialHooks/PreTrial',
        request_serializer=cogment_dot_api_dot_hooks__pb2.TrialContext.SerializeToString,
        response_deserializer=cogment_dot_api_dot_hooks__pb2.TrialContext.FromString,
        )
    self.Version = channel.unary_unary(
        '/cogment.TrialHooks/Version',
        request_serializer=cogment_dot_api_dot_common__pb2.VersionRequest.SerializeToString,
        response_deserializer=cogment_dot_api_dot_common__pb2.VersionInfo.FromString,
        )


class TrialHooksServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def PreTrial(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Version(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_TrialHooksServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'PreTrial': grpc.unary_unary_rpc_method_handler(
          servicer.PreTrial,
          request_deserializer=cogment_dot_api_dot_hooks__pb2.TrialContext.FromString,
          response_serializer=cogment_dot_api_dot_hooks__pb2.TrialContext.SerializeToString,
      ),
      'Version': grpc.unary_unary_rpc_method_handler(
          servicer.Version,
          request_deserializer=cogment_dot_api_dot_common__pb2.VersionRequest.FromString,
          response_serializer=cogment_dot_api_dot_common__pb2.VersionInfo.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'cogment.TrialHooks', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/environment.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/environment.proto',
  package='cogment',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1d\x63ogment/api/environment.proto\x12\x07\x63ogment\x1a\x18\x63ogment/api/common.proto\"x\n\x0eObservationSet\x12\x0f\n\x07tick_id\x18\x01 \x01(\x04\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12.\n\x0cobservations\x18\x03 \x03(\x0b\x32\x18.cogment.ObservationData\x12\x12\n\nactors_map\x18\x04 \x03(\x05\"\x1c\n\tActionSet\x12\x0f\n\x07\x61\x63tions\x18\x01 \x03(\x0c\"\x91\x01\n\x0f\x45nvStartRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12*\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x1a.cogment.EnvironmentConfig\x12*\n\x0ctrial_config\x18\x04 \x01(\x0b\x32\x14.cogment.TrialConfig\x12\x14\n\x0c\x61\x63tor_counts\x18\x03 \x03(\x05\"A\n\rEnvStartReply\x12\x30\n\x0fobservation_set\x18\x01 \x01(\x0b\x32\x17.cogment.ObservationSet\"!\n\rEnvEndRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\"\r\n\x0b\x45nvEndReply\"i\n\x10\x45nvUpdateRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12&\n\naction_set\x18\x02 \x01(\x0b\x32\x12.cogment.ActionSet\x12\x1b\n\x13reply_with_snapshot\x18\x03 \x01(\x08\"\x9f\x01\n\x0e\x45nvUpdateReply\x12\x30\n\x0fobservation_set\x18\x01 \x01(\x0b\x32\x17.cogment.ObservationSet\x12$\n\tfeedbacks\x18\x02 \x03(\x0b\x32\x11.cogment.Feedback\x12\x11\n\tend_trial\x18\x03 \x01(\x08\x12\"\n\x08messages\x18\x04 \x03(\x0b\x32\x10.cogment.Message\"]\n\x13\x45nvOnMessageRequest\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12\x10\n\x08\x61\x63tor_id\x18\x02 \x01(\x05\x12\"\n\x08messages\x18\x03 \x03(\x0b\x32\x10.cogment.Message\"\x13\n\x11\x45nvOnMessageReply2\xc6\x02\n\x0b\x45nvironment\x12;\n\x05Start\x12\x18.cogment.EnvStartRequest\x1a\x16.cogment.EnvStartReply\"\x00\x12\x35\n\x03\x45nd\x12\x16.cogment.EnvEndRequest\x1a\x14.cogment.EnvEndReply\"\x00\x12>\n\x06Update\x12\x19.cogment.EnvUpdateRequest\x1a\x17.cogment.EnvUpdateReply\"\x00\x12:\n\x07Version\x12\x17.cogment.VersionRequest\x1a\x14.cogment.VersionInfo\"\x00\x12G\n\tOnMessage\x12\x1c.cogment.EnvOnMessageRequest\x1a\x1a.cogment.EnvOnMessageReply\"\x00\x62\x06proto3')
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,])




_OBSERVATIONSET = _descriptor.Descriptor(
  name='ObservationSet',
  full_name='cogment.ObservationSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogment.ObservationSet.tick_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogment.ObservationSet.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='observations', full_name='cogment.ObservationSet.observations', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actors_map', full_name='cogment.ObservationSet.actors_map', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=68,
  serialized_end=188,
)


_ACTIONSET = _descriptor.Descriptor(
  name='ActionSet',
  full_name='cogment.ActionSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='actions', full_name='cogment.ActionSet.actions', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=218,
)


_ENVSTARTREQUEST = _descriptor.Descriptor(
  name='EnvStartRequest',
  full_name='cogment.EnvStartRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.EnvStartRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogment.EnvStartRequest.config', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trial_config', full_name='cogment.EnvStartRequest.trial_config', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_counts', full_name='cogment.EnvStartRequest.actor_counts', index=3,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=221,
  serialized_end=366,
)


_ENVSTARTREPLY = _descriptor.Descriptor(
  name='EnvStartReply',
  full_name='cogment.EnvStartReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='observation_set', full_name='cogment.EnvStartReply.observation_set', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=368,
  serialized_end=433,
)


_ENVENDREQUEST = _descriptor.Descriptor(
  name='EnvEndRequest',
  full_name='cogment.EnvEndRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.EnvEndRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=435,
  serialized_end=468,
)


_ENVENDREPLY = _descriptor.Descriptor(
  name='EnvEndReply',
  full_name='cogment.EnvEndReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=470,
  serialized_end=483,
)


_ENVUPDATEREQUEST = _descriptor.Descriptor(
  name='EnvUpdateRequest',
  full_name='cogment.EnvUpdateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.EnvUpdateRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='action_set', full_name='cogment.EnvUpdateRequest.action_set', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reply_with_snapshot', full_name='cogment.EnvUpdateRequest.reply_with_snapshot', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=485,
  serialized_end=590,
)


_ENVUPDATEREPLY = _descriptor.Descriptor(
  name='EnvUpdateReply',
  full_name='cogment.EnvUpdateReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='observation_set', full_name='cogment.EnvUpdateReply.observation_set', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feedbacks', full_name='cogment.EnvUpdateReply.feedbacks', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_trial', full_name='cogment.EnvUpdateReply.end_trial', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.EnvUpdateReply.messages', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=593,
  serialized_end=752,
)


_ENVONMESSAGEREQUEST = _descriptor.Descriptor(
  name='EnvOnMessageRequest',
  full_name='cogment.EnvOnMessageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_id', full_name='cogment.EnvOnMessageRequest.trial_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='cogment.EnvOnMessageRequest.actor_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messages', full_name='cogment.EnvOnMessageRequest.messages', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=754,
  serialized_end=847,
)


_ENVONMESSAGEREPLY = _descriptor.Descriptor(
  name='EnvOnMessageReply',
  full_name='cogment.EnvOnMessageReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=849,
  serialized_end=868,
)

_OBSERVATIONSET.fields_by_name['observations'].message_type = cogment_dot_api_dot_common__pb2._OBSERVATIONDATA
_ENVSTARTREQUEST.fields_by_name['config'].message_type = cogment_dot_api_dot_common__pb2._ENVIRONMENTCONFIG
_ENVSTARTREQUEST.fields_by_name['trial_config'].message_type = cogment_dot_api_dot_common__pb2._TRIALCONFIG
_ENVSTARTREPLY.fields_by_name['observation_set'].message_type = _OBSERVATIONSET
_ENVUPDATEREQUEST.fields_by_name['action_set'].message_type = _ACTIONSET
_ENVUPDATEREPLY.fields_by_name['observation_set'].message_type = _OBSERVATIONSET
_ENVUPDATEREPLY.fields_by_name['feedbacks'].message_type = cogment_dot_api_dot_common__pb2._FEEDBACK
_ENVUPDATEREPLY.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
_ENVONMESSAGEREQUEST.fields_by_name['messages'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
DESCRIPTOR.message_types_by_name['ObservationSet'] = _OBSERVATIONSET
DESCRIPTOR.message_types_by_name['ActionSet'] = _ACTIONSET
DESCRIPTOR.message_types_by_name['EnvStartRequest'] = _ENVSTARTREQUEST
DESCRIPTOR.message_types_by_name['EnvStartReply'] = _ENVSTARTREPLY
DESCRIPTOR.message_types_by_name['EnvEndRequest'] = _ENVENDREQUEST
DESCRIPTOR.message_types_by_name['EnvEndReply'] = _ENVENDREPLY
DESCRIPTOR.message_types_by_name['EnvUpdateRequest'] = _ENVUPDATEREQUEST
DESCRIPTOR.message_types_by_name['EnvUpdateReply'] = _ENVUPDATEREPLY
DESCRIPTOR.message_types_by_name['EnvOnMessageRequest'] = _ENVONMESSAGEREQUEST
DESCRIPTOR.message_types_by_name['EnvOnMessageReply'] = _ENVONMESSAGEREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObservationSet = _reflection.GeneratedProtocolMessageType('ObservationSet', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATIONSET,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.ObservationSet)
  })
_sym_db.RegisterMessage(ObservationSet)

ActionSet = _reflection.GeneratedProtocolMessageType('ActionSet', (_message.Message,), {
  'DESCRIPTOR' : _ACTIONSET,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.ActionSet)
  })
_sym_db.RegisterMessage(ActionSet)

EnvStartRequest = _reflection.GeneratedProtocolMessageType('EnvStartRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENVSTARTREQUEST,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvStartRequest)
  })
_sym_db.RegisterMessage(EnvStartRequest)

EnvStartReply = _reflection.GeneratedProtocolMessageType('EnvStartReply', (_message.Message,), {
  'DESCRIPTOR' : _ENVSTARTREPLY,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvStartReply)
  })
_sym_db.RegisterMessage(EnvStartReply)

EnvEndRequest = _reflection.GeneratedProtocolMessageType('EnvEndRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENVENDREQUEST,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvEndRequest)
  })
_sym_db.RegisterMessage(EnvEndRequest)

EnvEndReply = _reflection.GeneratedProtocolMessageType('EnvEndReply', (_message.Message,), {
  'DESCRIPTOR' : _ENVENDREPLY,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvEndReply)
  })
_sym_db.RegisterMessage(EnvEndReply)

EnvUpdateRequest = _reflection.GeneratedProtocolMessageType('EnvUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENVUPDATEREQUEST,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvUpdateRequest)
  })
_sym_db.RegisterMessage(EnvUpdateRequest)

EnvUpdateReply = _reflection.GeneratedProtocolMessageType('EnvUpdateReply', (_message.Message,), {
  'DESCRIPTOR' : _ENVUPDATEREPLY,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvUpdateReply)
  })
_sym_db.RegisterMessage(EnvUpdateReply)

EnvOnMessageRequest = _reflection.GeneratedProtocolMessageType('EnvOnMessageRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENVONMESSAGEREQUEST,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvOnMessageRequest)
  })
_sym_db.RegisterMessage(EnvOnMessageRequest)

EnvOnMessageReply = _reflection.GeneratedProtocolMessageType('EnvOnMessageReply', (_message.Message,), {
  'DESCRIPTOR' : _ENVONMESSAGEREPLY,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogment.EnvOnMessageReply)
  })
_sym_db.RegisterMessage(EnvOnMessageReply)



_ENVIRONMENT = _descriptor.ServiceDescriptor(
  name='Environment',
  full_name='cogment.Environment',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=871,
  serialized_end=1197,
  methods=[
  _descriptor.MethodDescriptor(
    name='Start',
    full_name='cogment.Environment.Start',
    index=0,
    containing_service=None,
    input_type=_ENVSTARTREQUEST,
    output_type=_ENVSTARTREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='End',
    full_name='cogment.Environment.End',
    index=1,
    containing_service=None,
    input_type=_ENVENDREQUEST,
    output_type=_ENVENDREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Update',
    full_name='cogment.Environment.Update',
    index=2,
    containing_service=None,
    input_type=_ENVUPDATEREQUEST,
    output_type=_ENVUPDATEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='cogment.Environment.Version',
    index=3,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._VERSIONREQUEST,
    output_type=cogment_dot_api_dot_common__pb2._VERSIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OnMessage',
    full_name='cogment.Environment.OnMessage',
    index=4,
    containing_service=None,
    input_type=_ENVONMESSAGEREQUEST,
    output_type=_ENVONMESSAGEREPLY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENVIRONMENT)

DESCRIPTOR.services_by_name['Environment'] = _ENVIRONMENT

# @@protoc_insertion_point(module_scope)



# General cogment error
class Error(Exception):
    pass


# Error that occured while configuring cogment
class ConfigError(Error):
    def __init__(self, expression, message):
        self.expression = expression
        self.message = message



from abc import ABC
import traceback

from types import SimpleNamespace
from typing import Dict

from cogment.api.hooks_pb2_grpc import TrialHooksServicer as Servicer
from cogment.api.hooks_pb2 import TrialContext
from cogment.api.common_pb2 import TrialParams
from cogment.utils import list_versions


class TrialHooks(ABC):
    VERSIONS: Dict[str, str]

    def before(self, trial_id, user_id, trial_params):
        pass


def _raw_params_to_user_params(params, settings):
    trial_config = None
    if params.HasField("trial_config"):
        trial_config = settings.trial.config_type()
        trial_config.ParseFromString(params.trial_config.content)

    env_config = None
    if(params.environment.HasField("config")):
        env_config = settings.environment.config_type()
        env_config.ParseFromString(params.environment.config.content)

    environment = SimpleNamespace(
        endpoint=params.environment.endpoint,
        config=env_config
    )

    actors = []

    for a in params.actors:
        actor_config = None

        if a.HasField("config"):
            a_c = settings.actor_classes.__getattribute__(a.actor_class)
            actor_config = a_c.config_type()
            actor_config.ParseFromString(a.config.content)

        actor = SimpleNamespace(
            actor_class=a.actor_class,
            endpoint=a.endpoint,
            config=actor_config
        )

        actors.append(actor)

    return SimpleNamespace(
        trial_config=trial_config,
        environment=environment,
        actors=actors,
        max_steps=params.max_steps,
        max_inactivity=params.max_inactivity
    )


def _user_params_to_raw_params(params, settings):
    result = TrialParams()

    result.max_steps = params.max_steps
    result.max_inactivity = params.max_inactivity

    if params.trial_config is not None:
        result.trial_config.content = params.trial_config.SerializeToString()

    result.environment.endpoint = params.environment.endpoint
    if params.environment.config is not None:
        result.environment.config.content = \
            params.environment.config.SerializeToString()

    for a in params.actors:
        actor_pb = result.actors.add()
        actor_pb.actor_class = a.actor_class
        actor_pb.endpoint = a.endpoint
        if a.config is not None:
            actor_pb.config.content = a.config.SerializeToString()

    return result


class HooksService(Servicer):
    def __init__(self, hooks_class, settings):
        self._hooks_class = hooks_class
        self.settings = settings
        self._hooks = hooks_class()

    def PreTrial(self, request, context):
        try:
            print(request)
            result = TrialContext()
            result.CopyFrom(request)

            user_params = _raw_params_to_user_params(request.params,
                                                     self.settings)
            user_params = self._hooks.pre_trial(request.trial_id,
                                                request.user_id,
                                                user_params)

            result.params.CopyFrom(_user_params_to_raw_params(user_params,
                                                              self.settings))

            return result
        except Exception:
            traceback.print_exc()
            raise

    def Version(self, request, context):
        try:
            return list_versions(self._hooks_class)
        except Exception:
            traceback.print_exc()
            raise



class ActorClass:
    def __init__(self, id, config_type, action_space, observation_space,
                 observation_delta, observation_delta_apply_fn, feedback_space, message_space):
        self.id = id
        self.config_type = config_type
        self.action_space = action_space
        self.observation_space = observation_space
        self.observation_delta = observation_delta
        self.observation_delta_apply_fn = observation_delta_apply_fn
        self.feedback_space = feedback_space
        self.message_space = message_space


class ActorClassList:
    def __init__(self, *args):
        self._actor_classes_list = list(args)

        for a_c in args:
            setattr(self, a_c.id, a_c)

    def __iter__(self):
        return iter(self._actor_classes_list)

    def get_actor_counts(self, trial_params):
        result = [0] * len(self._actor_classes_list)

        index_map = {}
        for index, actor_class in enumerate(self._actor_classes_list):
            index_map[actor_class.id] = index

        for actor in trial_params.actors:
            result[index_map[actor.actor_class]] += 1

        return result
